/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
/// <reference types="jest__types" />
import type { Config } from '@jest/types';
import type { HasteMapStatic, HasteRegExp, IHasteMap, IModuleMap } from './types';
declare type Options = {
    cacheDirectory?: string;
    computeDependencies?: boolean;
    computeSha1?: boolean;
    console?: Console;
    dependencyExtractor?: string | null;
    enableSymlinks?: boolean;
    extensions: Array<string>;
    forceNodeFilesystemAPI?: boolean;
    hasteImplModulePath?: string;
    hasteMapModulePath?: string;
    id: string;
    ignorePattern?: HasteRegExp;
    maxWorkers: number;
    mocksPattern?: string;
    platforms: Array<string>;
    resetCache?: boolean;
    retainAllFiles: boolean;
    rootDir: string;
    roots: Array<string>;
    skipPackageJson?: boolean;
    throwOnModuleCollision?: boolean;
    useWatchman?: boolean;
    watch?: boolean;
    workerThreads?: boolean;
};
export declare const ModuleMap: {
    create: (rootPath: string) => IModuleMap;
};
export type { IHasteFS, IHasteMap, IModuleMap, SerializableModuleMap, } from './types';
export declare class DuplicateError extends Error {
    mockPath1: string;
    mockPath2: string;
    constructor(mockPath1: string, mockPath2: string);
}
declare type IJestHasteMap = HasteMapStatic & {
    create(options: Options): Promise<IHasteMap>;
    getStatic(config: Config.ProjectConfig): HasteMapStatic;
};
declare const JestHasteMap: IJestHasteMap;
export default JestHasteMap;
