/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
import type { Config } from '@jest/types';
import type { SnapshotFormat } from './types';
export declare type SnapshotStateOptions = {
    readonly updateSnapshot: Config.SnapshotUpdateState;
    readonly prettierPath?: string | null;
    readonly expand?: boolean;
    readonly snapshotFormat: SnapshotFormat;
    readonly rootDir: string;
};
export declare type SnapshotMatchOptions = {
    readonly testName: string;
    readonly received: unknown;
    readonly key?: string;
    readonly inlineSnapshot?: string;
    readonly isInline: boolean;
    readonly error?: Error;
};
declare type SnapshotReturnOptions = {
    readonly actual: string;
    readonly count: number;
    readonly expected?: string;
    readonly key: string;
    readonly pass: boolean;
};
declare type SaveStatus = {
    deleted: boolean;
    saved: boolean;
};
export default class SnapshotState {
    private _counters;
    private _dirty;
    private _index;
    private readonly _updateSnapshot;
    private _snapshotData;
    private readonly _initialData;
    private readonly _snapshotPath;
    private _inlineSnapshots;
    private readonly _uncheckedKeys;
    private readonly _prettierPath;
    private readonly _rootDir;
    readonly snapshotFormat: SnapshotFormat;
    added: number;
    expand: boolean;
    matched: number;
    unmatched: number;
    updated: number;
    constructor(snapshotPath: string, options: SnapshotStateOptions);
    markSnapshotsAsCheckedForTest(testName: string): void;
    private _addSnapshot;
    clear(): void;
    save(): SaveStatus;
    getUncheckedCount(): number;
    getUncheckedKeys(): Array<string>;
    removeUncheckedKeys(): void;
    match({ testName, received, key, inlineSnapshot, isInline, error, }: SnapshotMatchOptions): SnapshotReturnOptions;
    fail(testName: string, _received: unknown, key?: string): string;
}
export {};
